#include <stdio.h>
#include <stdlib.h>  // exit()
#include <unistd.h>  // close()
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <string.h>  // strerror()
#include <signal.h>
#include <errno.h>  // errno
#include <fcntl.h>

typedef unsigned char U8;
typedef signed char S8;
typedef unsigned short int U16;
typedef signed short int S16;
typedef unsigned int U32;
typedef signed int S32;
typedef long L32;
typedef long long L64;
typedef float F32;
typedef double F64;

//--------------------------------------------------------------------
// Macro definition
//--------------------------------------------------------------------
#define PACKET_BUFFER_SIZE 1024*32
char usbtmc_packet_buff[PACKET_BUFFER_SIZE];

#define DEST_BUFFER_SIZE 1024*32
char usbtmc_dest_buff[DEST_BUFFER_SIZE];

//--------------------------------------------------------------------
// Function prototype
//--------------------------------------------------------------------
void signal_handler(int sig);
int dataCheck(U8* data, S32 size);

//--------------------------------------------------------------------
// Function implementation
//--------------------------------------------------------------------

int main(int argc, char **argv) {
    int fd;
    char cmd_str[] = ":WAV:DATA?\r\n";
    fd_set readfd;
    int retSize;
    int receivedTotal = 0;
    char tp, num;
    int ret;
    int fileSize;
    int received;
    int check;
    char *dataSrc;
    int dataLen;

    fd = open("/dev/usbtmc0", O_RDWR);
    if (fd <= 0) {
        printf("%s\n", "open usbtmc0 USB_FAIL");
        return;
    }

    while (1) {
        if (write(fd, cmd_str, sizeof (cmd_str)) < 0) {
            printf("write() failed: %s\n", strerror(errno));
        }

        //printf("usbtmc0 send %s\n", cmd_str);
        printf("usbtmc0 send\n");

        retSize = 0;
        receivedTotal = 0;
        while (1) {
            retSize = read(fd, usbtmc_packet_buff, PACKET_BUFFER_SIZE);
            printf("retSize = %d\n", retSize);
            if (usbtmc_packet_buff[0] == '#' && receivedTotal == 0) {
                num = usbtmc_packet_buff[1] - '0';
                tp = usbtmc_packet_buff[num + 2];
                usbtmc_packet_buff[num + 2] = 0;

                printf("---num=%d,rece=%s---\n", num, usbtmc_packet_buff);
                fileSize = atoi(&usbtmc_packet_buff[2]);
                usbtmc_packet_buff[num + 2] = tp;
                printf("---filesize=%d---\n", fileSize);

                dataSrc = &usbtmc_packet_buff[num + 2];
                dataLen = retSize - (num + 2);
            } else {
                dataSrc = usbtmc_packet_buff;
                dataLen = retSize;
            }

            if (dataLen > 0) {
                memcpy(&usbtmc_dest_buff[receivedTotal], dataSrc, dataLen);
            }

            receivedTotal += dataLen;

            if (receivedTotal >= fileSize) {
                break;
            }

            usleep(1000);
        }

        //TODO: 
        check = dataCheck(usbtmc_dest_buff, fileSize);
        printf("check=%d\n", check);
        usleep(1000 * 1000);
    }

    close(fd);

    return 0;
}

void signal_handler(int sig) {
    fprintf(stdout, "Caught signal %d\n", sig);
}


//----------------------for check-------------------------

#define START_VALUE     0X0A0A0550 
#define CHECK_VALUE     0X09090606
#define STARTEND_VALUE  0X09060609 
#define CHECKEND_VALUE  0X05A0050A 

#define OFFSET_START            0   //同步开始
#define OFFSET_CHECK            4   //校验
#define OFFSET_DYNAMIC_CHECK    8   //动态校验
#define OFFSET_INFO_SIZE        10  //参数信息占用字节数
#define OFFSET_RUN_STATUS       12  //运行状态
#define OFFSET_ADC_PRECISION    14  //垂直分辨率
#define OFFSET_CH_NUMS          16  //数据中波形通道数
#define OFFSET_WAVE_DATA_SIZE   18  //每个通道波形数据点个数      
#define OFFSET_FRAME_NUMS       22  //波形叠加次数
#define OFFSET_FFT_VALID        24  //FFT数据有无
#define OFFSET_FFT_SIZE         26  //FFT原始数据点个数
#define OFFSET_DRAW_MODE        30  //画图模式,0:单点平铺,1:压缩画图,0xff:深存储数据
#define OFFSET_ROLL_MODE        32  //滚动模式,0：正常扫描,1:滚动模式
#define OFFSET_ROLL_DATA_SIZE   34  //滚动数据个数

#define OFFSET_FREQ_CH1         38
#define OFFSET_FREQ_CH2         42
#define OFFSET_FREQ_CH3         46
#define OFFSET_FREQ_CH4         50
#define OFFSET_REF_FREQ_CH1     54
#define OFFSET_REF_FREQ_CH2     58
#define OFFSET_REF_FREQ_CH3     62
#define OFFSET_REF_FREQ_CH4     66  

#define OFFSET_ADC_OVER_FLAG    70

#define OFFSET_ADC_MIN_CH1      72
#define OFFSET_ADC_MIN_CH2      74
#define OFFSET_ADC_MIN_CH3      76
#define OFFSET_ADC_MIN_CH4      78    
#define OFFSET_ADC_MAX_CH1      80
#define OFFSET_ADC_MAX_CH2      82
#define OFFSET_ADC_MAX_CH3      84
#define OFFSET_ADC_MAX_CH4      86
#define OFFSET_ADC_AVG_CH1      88
#define OFFSET_ADC_AVG_CH2      90
#define OFFSET_ADC_AVG_CH3      92
#define OFFSET_ADC_AVG_CH4      94    

#define OFFSET_TRIG_TYPE        96

#define OFFSET_VOLTSCALE_CH1    260
#define OFFSET_VOLTSCALE_CH2    262
#define OFFSET_VOLTSCALE_CH3    264
#define OFFSET_VOLTSCALE_CH4    266

#define OFFSET_ZERO_CH1         268
#define OFFSET_ZERO_CH2         272
#define OFFSET_ZERO_CH3         276
#define OFFSET_ZERO_CH4         280

typedef struct {
    U32 start;
    U32 check;
    U16 dynamicCheck;
    U16 infoSize;
    U16 runStatus;
    U16 adcPrecision;
    U16 chNums;
    U32 waveDataSize;
    U16 frameNums;
    U16 fftValid;
    U32 fftDataSize;
    U16 drawMode;
    U16 rollMode;
    U32 rollDataSize;

    U16 dynamicCheckEnd;
    U32 checkEnd;
    U32 startEnd;
} protocol_data_t;

protocol_data_t protocolData;

U32 getU32Data(U8 *data, int offset) {
    U32 value;
    U8 *p = (U8*) & value;

    *p++ = data[offset++];
    *p++ = data[offset++];
    *p++ = data[offset++];
    *p++ = data[offset++];

    return value;
}

U16 getU16Data(U8 *data, int offset) {
    U16 value;
    U8 *p = (U8*) & value;

    *p++ = data[offset++];
    *p++ = data[offset++];

    return value;
}

int getDynamicCheckEndOffset() {
    int offset;
    int N1 = protocolData.infoSize;
    int N2 = protocolData.frameNums * protocolData.chNums * (protocolData.waveDataSize * 2 + 2);
    int N3 = protocolData.fftValid * (protocolData.fftDataSize * 2 + 2);

    offset = N1 + N2 + N3 + 16;
    return offset;
}

int getCheckEndOffset() {
    int offset;

    int dynamicCheckEnd = getDynamicCheckEndOffset();

    offset = dynamicCheckEnd + 2;

    return offset;
}

int getStartEndOffset() {
    int offset;

    int dynamicCheckEnd = getDynamicCheckEndOffset();

    offset = dynamicCheckEnd + 6;

    return offset;
}

int dataCheck(U8* data, S32 size) {
    static int cnt = 0;
    int ch;
    protocolData.start = getU32Data(data, OFFSET_START);
    printf("start=0x%0x\n", protocolData.start);
    if (protocolData.start != START_VALUE) {
        return 0;
    }

    protocolData.check = getU32Data(data, OFFSET_CHECK);
    printf("check=0x%0x\n", protocolData.check);
    if (protocolData.check != CHECK_VALUE) {
        return 0;
    }

    protocolData.dynamicCheck = getU16Data(data, OFFSET_DYNAMIC_CHECK);
    protocolData.infoSize = getU16Data(data, OFFSET_INFO_SIZE);
    protocolData.runStatus = getU16Data(data, OFFSET_RUN_STATUS);
    protocolData.adcPrecision = getU16Data(data, OFFSET_ADC_PRECISION);
    protocolData.chNums = getU16Data(data, OFFSET_CH_NUMS);
    protocolData.waveDataSize = getU32Data(data, OFFSET_WAVE_DATA_SIZE);
    protocolData.frameNums = getU16Data(data, OFFSET_FRAME_NUMS);
    protocolData.fftValid = getU16Data(data, OFFSET_FFT_VALID);
    protocolData.fftDataSize = getU32Data(data, OFFSET_FFT_SIZE);
    protocolData.drawMode = getU16Data(data, OFFSET_DRAW_MODE);
    protocolData.rollMode = getU16Data(data, OFFSET_ROLL_MODE);
    protocolData.rollDataSize = getU32Data(data, OFFSET_ROLL_DATA_SIZE);




    printf("dynamicCheck=%d\n", protocolData.dynamicCheck);
    printf("infoSize=%d\n", protocolData.infoSize);
    printf("runStatus=%d\n", protocolData.runStatus);
    printf("adcPrecision=%d\n", protocolData.adcPrecision);
    printf("chNums=%d\n", protocolData.chNums);
    printf("waveDataSize=%d\n", protocolData.waveDataSize);
    printf("frameNums=%d\n", protocolData.frameNums);
    printf("fftValid=%d\n", protocolData.fftValid);
    printf("fftDataSize=%d\n", protocolData.fftDataSize);
    printf("drawMode=%d\n", protocolData.drawMode);
    printf("rollMode=%d\n", protocolData.rollMode);
    printf("rollDataSize=%d\n", protocolData.rollDataSize);



    protocolData.dynamicCheckEnd = getU32Data(data, getDynamicCheckEndOffset());
    printf("dynamicCheckEnd=%d\n", protocolData.dynamicCheckEnd);

    if (protocolData.dynamicCheckEnd != protocolData.dynamicCheck) {
        return 0;
    }

    protocolData.checkEnd = getU32Data(data, getCheckEndOffset());
    printf("checkEnd=0x%0x\n", protocolData.checkEnd);
    if (protocolData.checkEnd != CHECKEND_VALUE) {
        return 0;
    }
    protocolData.startEnd = getU32Data(data, getStartEndOffset());
    printf("startEnd=0x%0x\n", protocolData.startEnd);
    if (protocolData.startEnd != STARTEND_VALUE) {
        return 0;
    }

    return 1;
}